--------------------
-- Class: ColourPicker --
-- Usage:
--  local colorFields = ColourPicker:New{
--    parent        = parent,
--    anch          = "BOTTOMRIGHT",
--    x             = -25,
--    y             = 75,
--  }
--------------------

local setmetatable = setmetatable
local CreateFrame, UIParent, GetCursorPosition, tonumber = CreateFrame, UIParent, GetCursorPosition, tonumber
local strlower, select = strlower, select
local UnitClass = UnitClass

local font = "Fonts\\FRIZQT__.TTF"
local leftLabelMargin = 145
local itemColor = {r=0,g=0.5,b=1,a=1 }
local count = 1

local classColors = {
  ["deathknight"] = { r=0.77, g=0.12, b=0.23, a=1, },
  ["demonhunter"] = { r=0.64, g=0.19, b=0.79, a=1, },
  ["druid"]       = { r=1.00, g=0.49, b=0.04, a=1, },
  ["hunter"]      = { r=0.67, g=0.83, b=0.45, a=1, },
  ["mage"]        = { r=0.41, g=0.80, b=0.94, a=1, },
  ["monk"]        = { r=0.00, g=1.00, b=0.59, a=1, },
  ["paladin"]     = { r=0.96, g=0.55, b=0.73, a=1, },
  ["priest"]      = { r=1.00, g=1.00, b=1.00, a=1, },
  ["rogue"]       = { r=1.00, g=0.96, b=0.41, a=1, },
  ["shaman"]      = { r=0.00, g=0.44, b=0.87, a=1, },
  ["warlock"]     = { r=0.58, g=0.51, b=0.79, a=1, },
  ["warrior"]     = { r=0.78, g=0.61, b=0.43, a=1, },
  ["default"]     = { r=0.00, g=1.00, b=0.00, a=1, },
};

local mtButton = {
  parent  = UIParent,
  width   = 135,
  height  = 25,
  anch    = "TOPLEFT",
  x       = 0,
  y       = 0,
  text    = "",
  onClick = function() end,
}

local mtColorSelect = {
  parent        = UIParent,
  width         = 160,
  height        = 175,
  wheelWidth    = 120,
  barWidth      = 20,
  thumbWidth    = 14,
  anch          = "TOPLEFT",
  x             = 0,
  y             = 0,
}

local mtEdit = {
  parent    = UIParent,
  width     = 150,
  height    = 25,
  anch      = "TOPLEFT",
  x         = 0,
  y         = 0,
  letters   = 0,
  text      = "",
  autofocus = false,
}

local mtLabel = {
  parent   = UIParent,
  width    = 150,
  height   = 50,
  anch     = "TOPLEFT",
  x        = 0,
  y        = 0,
  text     = "",
  justifyV = "CENTER",
  justifyH = "CENTER",
  font     = font,
  fontSize = 19,
  color    = {r=1,g=1,b=1,a=1,},
}

local createButton = function(btn)
  btn = setmetatable(btn, { __index = mtButton });
  
  local button = CreateFrame("Button", btn.name, btn.parent, "GameMenuButtonTemplate")
  button:SetSize  (btn.width, btn.height)
  button:SetPoint (btn.anch, btn.x, btn.y)
  button:SetText  (btn.text)
  button:SetScript("OnClick", btn.onClick)
  
  return button
end

local addItemLabel = function(name, target, text, location, color)
  location = location or "left"
  color    = color    or {r=1,g=1,b=1,a=1,}
  
  -- Determine relative location
  local anch, x, y = select(3, target:GetPoint())
  x                = (strlower(location) == "left") and (x - leftLabelMargin) or ((strlower(location) == "top") and (x + 20) or x)
  y                = (strlower(location) == "left") and (y - 5)               or ((strlower(location) == "top") and (y + 15) or y)
  
  local fontString = target:GetParent():CreateFontString(name)
  fontString:SetFont     (font, 19)
  fontString:SetPoint    (anch,x,y)
  fontString:SetSize     (181, 39)
  fontString:SetTextColor(color.r, color.g, color.b, color.a)
  fontString:SetText     (text)
  fontString:SetJustifyV ("TOP")
  fontString:SetJustifyH ("Left")
  
  return fontString
end

local createEditBox = function(edit)
  edit = setmetatable(edit, { __index = mtEdit });
  
  local editBox = CreateFrame("EditBox", edit.name, edit.parent, "InputBoxTemplate")
  editBox:SetSize      (edit.width, edit.height)
  editBox:SetPoint     (edit.anch, edit.x, edit.y)
  editBox:SetMaxLetters(edit.letters)
  editBox:SetText      (edit.text or "")
  editBox:SetAutoFocus (edit.autofocus or false)
  
  if (edit.onEnterPressed) then
    editBox:SetScript("OnEnterPressed", edit.onEnterPressed)
  end
  
  if (edit.onTextChanged) then
    editBox:SetScript("OnTextChanged", edit.onTextChanged)
  end
  
  if edit.label then
    editBox.label = addItemLabel(edit.label.name, editBox, edit.label.text, edit.label.location, edit.label.color)
  end
  
  return editBox
end

local createLabel = function(label)
  label = setmetatable(label, { __index = mtLabel });
  
  local fontString = label.parent:CreateFontString(label.name)
  fontString:SetFont     (font, 19)
  fontString:SetPoint    (label.anch, label.x, label.y)
  fontString:SetSize     (label.width, label.height)
  fontString:SetTextColor(label.color.r, label.color.g, label.color.b, label.color.a)
  fontString:SetText     (label.text)
  fontString:SetJustifyV (label.justifyV)
  fontString:SetJustifyH (label.justifyH)
  
  return fontString
end

local ColourPicker = {};

function ColourPicker:New(colorSelect)
  colorSelect = setmetatable(colorSelect, { __index = mtColorSelect });
  
  -- Create the ColorSelect frame.
  local colorFields = CreateFrame("ColorSelect", "ColourPicker"..count, colorSelect.parent)
  colorFields:SetPoint(colorSelect.anch, colorSelect.x, colorSelect.y)
  colorFields:SetSize (colorSelect.width, colorSelect.height)
  
  -- Create colorpicker wheel.
  colorFields.cWheelTexture = colorFields:CreateTexture()
  colorFields.cWheelTexture:SetWidth (colorSelect.wheelWidth)
  colorFields.cWheelTexture:SetHeight(colorSelect.wheelWidth)
  colorFields.cWheelTexture:SetPoint ("TOPLEFT",0,0)
  colorFields:SetColorWheelTexture   (colorFields.cWheelTexture)
  
  colorFields.cWheelThumbTexture = colorFields:CreateTexture()
  colorFields.cWheelThumbTexture:SetTexture ("Interface\\Buttons\\UI-ColorPicker-Buttons")
  colorFields.cWheelThumbTexture:SetWidth   (colorSelect.thumbWidth)
  colorFields.cWheelThumbTexture:SetHeight  (colorSelect.thumbWidth)
  colorFields.cWheelThumbTexture:SetTexCoord(0, 0.15625, 0, 0.625)
  colorFields:SetColorWheelThumbTexture     (colorFields.cWheelThumbTexture)
  
  -- Create the colorpicker slider.
  colorFields.valueTexture = colorFields:CreateTexture()
  colorFields.valueTexture:SetWidth (colorSelect.barWidth)
  colorFields.valueTexture:SetHeight(colorSelect.wheelWidth)
  colorFields.valueTexture:SetPoint ("TOPRIGHT",-30 + colorSelect.barWidth, 0)
  colorFields:SetColorValueTexture  (colorFields.valueTexture)
  
  colorFields.valueThumbTexture = colorFields:CreateTexture()
  colorFields.valueThumbTexture:SetTexture ("Interface\\Buttons\\UI-ColorPicker-Buttons")
  colorFields.valueThumbTexture:SetWidth   (colorSelect.barWidth)
  colorFields.valueThumbTexture:SetHeight  (colorSelect.thumbWidth)
  colorFields.valueThumbTexture:SetTexCoord(0.25, 1, 0.875, 0)
  colorFields:SetColorValueThumbTexture    (colorFields.valueThumbTexture)
  
  -- Create the alpha widget.
  colorFields.alphaTextureContainer = CreateFrame("Frame", "ColourPicker"..count.."AlphaContainer", colorFields)
  colorFields.alphaTextureContainer:SetHeight      (colorSelect.wheelWidth)
  colorFields.alphaTextureContainer:SetWidth       (colorSelect.barWidth)
  colorFields.alphaTextureContainer:SetPoint       ("TOPRIGHT",colorSelect.barWidth * 2 - 20, 0)
  colorFields.alphaTextureContainer:RegisterForDrag("LeftButton")
  colorFields.alphaTextureContainer:EnableMouse    (true)
  
  colorFields:SetScript("OnColorSelect", function(self)
    -- Ensure custom fields are updated.
    local r, g, b, a = self:GetColorRGBA()
    self.colorFieldA :SetText(100 * a)
    self.colorFieldR :SetText(r * 255)
    self.colorFieldG :SetText(g * 255)
    self.colorFieldB :SetText(b * 255)
    self.alphaTexture:SetGradientAlpha("VERTICAL",0,0,0,1,r,g,b,1)
  end)
  
  function colorFields:SetColorRGBA(r, g, b, a)
    self:SetColorAlpha(a)
    self:SetColorRGB(r,g,b)
  end
  
  function colorFields:GetColorRGBA()
    local r, g, b = self:GetColorRGB()
    return r, g, b, self:GetColorAlpha()
  end
  
  function colorFields:SetColorAlpha(a)
    a = a < 0 and 0 or a
    a = a > 1 and 1 or a
    
    -- Update Alpha slider and trigger OnColorSelect.
    self.alphaThumbTexture:SetPoint("TOPRIGHT",0,7 - (120 - 120 * a))
    self.colorFieldA:SetText(100 * a)
    self:SetColorRGB(self:GetColorRGB())
  end
  
  function colorFields:GetColorAlpha()
    local a = tonumber(self.colorFieldA:GetText()) or 100
    return a/100
  end
  
  colorFields.alphaTextureContainer:SetScript("OnDragStart", function(self)
    colorFields.alphaThumbContainer.minY = self:GetBottom()
  end)
  colorFields.alphaTextureContainer:SetScript("OnDragStop", function(self)
    colorFields.alphaThumbContainer.minY = nil
  end)
  
  colorFields.alphaTexture = colorFields.alphaTextureContainer:CreateTexture()
  colorFields.alphaTexture:SetAllPoints    (colorFields.alphaTextureContainer)
  colorFields.alphaTexture:SetColorTexture (1,1,1,1)
  colorFields.alphaTexture:SetGradientAlpha("VERTICAL",0,0,0,1,1,1,1,1)
  
  colorFields.alphaThumbTexture = colorFields.alphaTextureContainer:CreateTexture()
  colorFields.alphaThumbTexture:SetTexture ("Interface\\Buttons\\UI-ColorPicker-Buttons")
  colorFields.alphaThumbTexture:SetHeight  (colorSelect.thumbWidth)
  colorFields.alphaThumbTexture:SetWidth   (colorSelect.barWidth)
  colorFields.alphaThumbTexture:SetPoint   ("TOPRIGHT", 0, colorSelect.thumbWidth/2)
  colorFields.alphaThumbTexture:SetTexCoord(0.275, 1, 0.875, 0)
  
  -- Add the alpha slider.
  colorFields.alphaThumbContainer = CreateFrame("Frame", "ColourPicker"..count.."AlphaDragFrame", colorFields)
  
  colorFields.alphaThumbContainer:SetScript("OnUpdate", function(self)
    if self.minY then
      -- Calculate yPos relative to container bottom.
      local _,yPos = GetCursorPosition()
      yPos = yPos/UIParent:GetScale() - self.minY
      
      -- Ensure movement is within the container boundaries.
      yPos = (yPos < 0)   and 0   or yPos
      yPos = (yPos > 120) and 120 or yPos
      yPos = yPos + colorSelect.thumbWidth/2
      
      -- Drag the slider.
      colorFields.alphaThumbTexture:SetPoint("TOPRIGHT",0,yPos - colorSelect.wheelWidth)
      
      -- Update the alpha value.
      yPos = (yPos - colorSelect.thumbWidth/2) / 120
      colorFields:SetColorAlpha(yPos)
    end
  end)
  
  -- Create the edit fields
  local height = 25
  local width  = 30
  local xPos   = -width - 10
  local yPos   = height - 5
  
  -- Create the R color field
  colorFields.colorFieldR = createEditBox{
    parent        = colorFields,
    name          = "ColourFieldR"..count,
    width         = width,
    height        = height,
    anch          = "BOTTOMRIGHT",
    x             = (xPos*3),
    y             = 0,
    letters       = 3,
    onTextChanged = function(self)
      local val = self:GetText()
      if tonumber(val) then
        local r,g,b = colorFields:GetColorRGB()
        colorFields:SetColorRGB(val/255,g,b)
      end
      
      if colorSelect.onTextChanged then
        colorSelect.onTextChanged(self)
      end
    end,
  }
  
  -- Create the R color label
  colorFields.colorFieldR.label = createLabel{
    parent = colorFields,
    name   = "ColourFieldRLabel"..count,
    width  = width,
    height = height,
    anch   = "BOTTOMRIGHT",
    x      = (xPos*3) - 10,
    y      = yPos,
    color  = itemColor,
    text   = "R",
  }
  
  -- Create the G color field
  colorFields.colorFieldG = createEditBox{
    parent        = colorFields,
    name          = "ColourFieldG"..count,
    width         = width,
    height        = height,
    anch          = "BOTTOMRIGHT",
    x             = (xPos*2),
    y             = 0,
    letters       = 3,
    onTextChanged = function(self)
      local val = self:GetText()
      if tonumber(val) then
        local r,g,b = colorFields:GetColorRGB()
        colorFields:SetColorRGB(r,val/255,b)
      end
      
      if colorSelect.onTextChanged then
        colorSelect.onTextChanged(self)
      end
    end,
  }
  
  -- Create the G color label
  colorFields.colorFieldG.label = createLabel{
    parent = colorFields,
    name   = "ColourFieldGLabel"..count,
    width  = width,
    height = height,
    anch   = "BOTTOMRIGHT",
    x      = (xPos*2) - 10,
    y      = yPos,
    color  = itemColor,
    text   = "G",
  }
  
  -- Create the B color field
  colorFields.colorFieldB = createEditBox{
    parent        = colorFields,
    name          = "ColourFieldB"..count,
    width         = width,
    height        = height,
    anch          = "BOTTOMRIGHT",
    x             = xPos,
    y             = 0,
    letters       = 3,
    onTextChanged = function(self)
      local val = self:GetText()
      if tonumber(val) then
        local r,g,b = colorFields:GetColorRGB()
        colorFields:SetColorRGB(r,g,val/255)
      end
      
      if colorSelect.onTextChanged then
        colorSelect.onTextChanged(self)
      end
    end,
  }
  
  -- Create the B color label
  colorFields.colorFieldB.label = createLabel{
    parent = colorFields,
    name   = "ColourFieldBLabel"..count,
    width  = width,
    height = height,
    anch   = "BOTTOMRIGHT",
    x      = xPos - 10,
    y      = yPos,
    color  = itemColor,
    text   = "B",
  }
  
  -- Create the Alpha field
  colorFields.colorFieldA = createEditBox{
    parent        = colorFields,
    name          = "ColourFieldA"..count,
    width         = width,
    height        = height,
    anch          = "BOTTOMRIGHT",
    x             = 0,
    y             = 0,
    letters       = 3,
    onTextChanged = function(self)
      local val = self:GetText()
      if tonumber(val) then
        colorFields:SetColorAlpha(val/100)
      end
      
      if colorSelect.onTextChanged then
        colorSelect.onTextChanged(self)
      end
    end,
  }
  
  -- Create the Alpha label
  colorFields.colorFieldA.label = createLabel{
    parent = colorFields,
    name   = "ColourFieldALabel"..count,
    width  = width,
    height = height,
    anch   = "BOTTOMRIGHT",
    x      = -10,
    y      = yPos,
    color  = itemColor,
    text   = "A",
  }
  
  -- Add the reset button
  colorFields.classButton = createButton{
    parent  = colorFields,
    name    = "ColourFieldClassButton"..count,
    width   = 40,
    height  = height,
    anch    = "BOTTOMLEFT",
    x       = -40,
    y       = 0,
    text    = "Class",
    onClick = function()
      local class = strlower(select(2, UnitClass("player")))
      colorFields:SetColorRGBA(classColors[class].r, classColors[class].g, classColors[class].b, classColors[class].a)
    end,
  }
  
  return colorFields
end

-- Provide global access.
rawset(_G, "ColourPicker", ColourPicker)
